# frozen_string_literal: true

module Repubmark
  module Elems
    class Caption < Base
      include Joint::ForwardingBuilders

      parents :Blockquote, :Figure, :ListItem, :Paragraph

      def initialize(parent)
        super parent
        @items = []
      end

      #################
      # Basic methods #
      #################

      def word_count = @items.sum(&:word_count)

      def to_summary_plain
        @items.map(&:to_summary_plain).join(' ').freeze unless @items.empty?
      end

      def to_html
        return if @items.empty?

        [
          '<span>',
          *@items.map(&:to_html),
          '</span>',
        ].map { |s| "#{s}\n" }.join.freeze
      end

      def to_gemtext
        @items.map(&:to_gemtext).join(' ').strip.freeze unless @items.empty?
      end

      ##################
      # Helper methods #
      ##################

      def empty? = @items.empty?

      ###################
      # Builder methods #
      ###################

      def joint
        joint = Joint.new self
        yield joint
        @items << joint
        nil
      end

      def quote(str = nil)
        quote = Quote.new self
        case [!!str, block_given?]
        when [true, false] then quote.text str
        when [false, true] then yield quote
        else
          raise 'Invalid args'
        end
        @items << quote
        nil
      end
    end
  end
end
